Method: Bundler::Plugin::API.command

Defined in:
lib/bundler/plugin/api.rb

.command(command, cls = self) ⇒ Object

The plugins should declare that they handle a command through this helper.

Parameters:

  • command (String)

    being handled by them

  • (optional) (Class)

    class that handles the command. If not provided, the self class will be used.



33
34
35
# File 'lib/bundler/plugin/api.rb', line 33

def self.command(command, cls = self)
  Plugin.add_command command, cls
end