The plugins should declare that they provide a installation source through this helper.
Parameters:
source type they provide
class that handles the source. If not provided, the self class will be used.
self
43 44 45 46
# File 'lib/bundler/plugin/api.rb', line 43 def self.source(source, cls = self) cls.send :include, Bundler::Plugin::API::Source Plugin.add_source source, cls end