Method: Gem::Net::HTTP::Persistent#shutdown

Defined in:
lib/bundler/vendor/net-http-persistent/lib/net/http/persistent.rb

#shutdownObject

Shuts down all connections. Attempting to checkout a connection after shutdown will raise an error.

NOTE: Calling shutdown for can be dangerous!

If any thread is still using a connection it may cause an error! Call #shutdown when you are completely done making requests!



990
991
992
# File 'lib/bundler/vendor/net-http-persistent/lib/net/http/persistent.rb', line 990

def shutdown
  @pool.shutdown { |http| http.finish }
end