Module: Bundler
- Defined in:
- lib/bundler.rb,
lib/bundler/ui.rb,
lib/bundler/cli.rb,
lib/bundler/dsl.rb,
lib/bundler/env.rb,
lib/bundler/index.rb,
lib/bundler/retry.rb,
lib/bundler/digest.rb,
lib/bundler/errors.rb,
lib/bundler/mirror.rb,
lib/bundler/plugin.rb,
lib/bundler/source.rb,
lib/bundler/worker.rb,
lib/bundler/cli/add.rb,
lib/bundler/cli/gem.rb,
lib/bundler/fetcher.rb,
lib/bundler/runtime.rb,
lib/bundler/version.rb,
lib/bundler/checksum.rb,
lib/bundler/cli/exec.rb,
lib/bundler/cli/fund.rb,
lib/bundler/cli/info.rb,
lib/bundler/cli/init.rb,
lib/bundler/cli/list.rb,
lib/bundler/cli/lock.rb,
lib/bundler/cli/open.rb,
lib/bundler/cli/show.rb,
lib/bundler/injector.rb,
lib/bundler/resolver.rb,
lib/bundler/ruby_dsl.rb,
lib/bundler/settings.rb,
lib/bundler/spec_set.rb,
lib/bundler/ui/shell.rb,
lib/bundler/cli/cache.rb,
lib/bundler/cli/check.rb,
lib/bundler/cli/clean.rb,
lib/bundler/cli/issue.rb,
lib/bundler/constants.rb,
lib/bundler/deprecate.rb,
lib/bundler/installer.rb,
lib/bundler/ui/silent.rb,
lib/bundler/cli/common.rb,
lib/bundler/cli/config.rb,
lib/bundler/cli/doctor.rb,
lib/bundler/cli/plugin.rb,
lib/bundler/cli/remove.rb,
lib/bundler/cli/update.rb,
lib/bundler/definition.rb,
lib/bundler/dependency.rb,
lib/bundler/gem_helper.rb,
lib/bundler/plugin/api.rb,
lib/bundler/plugin/dsl.rb,
lib/bundler/source/git.rb,
lib/bundler/source_map.rb,
lib/bundler/ci_detector.rb,
lib/bundler/cli/console.rb,
lib/bundler/cli/install.rb,
lib/bundler/source/path.rb,
lib/bundler/source_list.rb,
lib/bundler/ui/rg_proxy.rb,
lib/bundler/cli/binstubs.rb,
lib/bundler/cli/outdated.rb,
lib/bundler/cli/platform.rb,
lib/bundler/cli/pristine.rb,
lib/bundler/current_ruby.rb,
lib/bundler/feature_flag.rb,
lib/bundler/fetcher/base.rb,
lib/bundler/plugin/index.rb,
lib/bundler/process_lock.rb,
lib/bundler/ruby_version.rb,
lib/bundler/safe_marshal.rb,
lib/bundler/self_manager.rb,
lib/bundler/vendored_uri.rb,
lib/bundler/fetcher/index.rb,
lib/bundler/plugin/events.rb,
lib/bundler/resolver/base.rb,
lib/bundler/resolver/root.rb,
lib/bundler/vendored_thor.rb,
lib/bundler/build_metadata.rb,
lib/bundler/cli/doctor/ssl.rb,
lib/bundler/force_platform.rb,
lib/bundler/match_metadata.rb,
lib/bundler/match_platform.rb,
lib/bundler/shared_helpers.rb,
lib/bundler/source/gemspec.rb,
lib/bundler/uri_normalizer.rb,
lib/bundler/vendored_tsort.rb,
lib/bundler/friendly_errors.rb,
lib/bundler/lockfile_parser.rb,
lib/bundler/materialization.rb,
lib/bundler/source/metadata.rb,
lib/bundler/source/rubygems.rb,
lib/bundler/yaml_serializer.rb,
lib/bundler/plugin/installer.rb,
lib/bundler/resolver/package.rb,
lib/bundler/plugin/api/source.rb,
lib/bundler/resolver/strategy.rb,
lib/bundler/vendored_pub_grub.rb,
lib/bundler/fetcher/dependency.rb,
lib/bundler/fetcher/downloader.rb,
lib/bundler/lazy_specification.rb,
lib/bundler/lockfile_generator.rb,
lib/bundler/plugin/source_list.rb,
lib/bundler/resolver/candidate.rb,
lib/bundler/settings/validator.rb,
lib/bundler/stub_specification.rb,
lib/bundler/vendored_fileutils.rb,
lib/bundler/cli/doctor/diagnose.rb,
lib/bundler/resolver/spec_group.rb,
lib/bundler/vendored_persistent.rb,
lib/bundler/compact_index_client.rb,
lib/bundler/gem_version_promoter.rb,
lib/bundler/installer/standalone.rb,
lib/bundler/plugin/installer/git.rb,
lib/bundler/remote_specification.rb,
lib/bundler/rubygems_integration.rb,
lib/bundler/source/git/git_proxy.rb,
lib/bundler/environment_preserver.rb,
lib/bundler/fetcher/compact_index.rb,
lib/bundler/match_remote_metadata.rb,
lib/bundler/plugin/installer/path.rb,
lib/bundler/source/path/installer.rb,
lib/bundler/endpoint_specification.rb,
lib/bundler/rubygems_gem_installer.rb,
lib/bundler/source/rubygems/remote.rb,
lib/bundler/uri_credentials_filter.rb,
lib/bundler/installer/gem_installer.rb,
lib/bundler/resolver/incompatibility.rb,
lib/bundler/plugin/installer/rubygems.rb,
lib/bundler/source/rubygems_aggregate.rb,
lib/bundler/vendor/uri/lib/uri/common.rb,
lib/bundler/compact_index_client/cache.rb,
lib/bundler/fetcher/gem_remote_fetcher.rb,
lib/bundler/compact_index_client/parser.rb,
lib/bundler/compact_index_client/updater.rb,
lib/bundler/installer/parallel_installer.rb,
lib/bundler/compact_index_client/cache_file.rb
Overview
Defined Under Namespace
Modules: BuildMetadata, CIDetector, Digest, FetchMetadata, FileUtils, ForcePlatform, FriendlyErrors, MatchMetadata, MatchPlatform, MatchRemoteMetadata, Persistent, Plugin, PubGrub, RubyDsl, SafeMarshal, SecureRandom, SharedHelpers, TSort, UI, URI, URICredentialsFilter, URINormalizer, YAMLSerializer
Classes: APIResponseInvalidDependenciesError, APIResponseMismatchError, BundlerError, CLI, Checksum, ChecksumMismatchError, CompactIndexClient, ConnectionPool, CorruptBundlerInstallError, CurrentRuby, CyclicDependencyError, Definition, Dependency, Deprecate, DeprecatedError, DirectoryRemovalError, Dsl, EndpointSpecification, Env, EnvironmentPreserver, FeatureFlag, Fetcher, GemHelper, GemInstaller, GemNotFound, GemRequireError, GemVersionPromoter, GemfileError, GemfileEvalError, GemfileLockNotFound, GemfileNotFound, GemspecError, GenericSystemCallError, GitError, HTTPError, IncorrectLockfileDependencies, Index, Injector, InsecureInstallPathError, InstallError, InstallHookError, Installer, InvalidArgumentError, InvalidOption, LazySpecification, LockfileError, LockfileGenerator, LockfileParser, MarshalError, Materialization, MirrorSockets, NoSpaceOnDeviceError, OperationNotPermittedError, OperationNotSupportedError, ParallelInstaller, PathError, PermissionError, PluginError, ProcessLock, ProductionError, ReadOnlyFileSystemError, RemoteSpecification, RemovedError, Resolver, Retry, RubyGemsGemInstaller, RubyVersion, RubyVersionMismatch, RubygemsIntegration, Runtime, SecurityError, SelfManager, Settings, SocketAddress, SolveFailure, Source, SourceList, SourceMap, SpecSet, Standalone, StubSpecification, TemporaryResourceError, Thor, ThreadCreationError, VirtualProtocolError, Worker, YamlSyntaxError
Constant Summary
collapse
- ORIGINAL_ENV =
environment_preserver.restore
- SUDO_MUTEX =
Thread::Mutex.new
- VERSION =
"4.0.12".freeze
- WINDOWS =
RbConfig::CONFIG["host_os"] =~ /(msdos|mswin|djgpp|mingw)/
- FREEBSD =
RbConfig::CONFIG["host_os"].to_s.include?("bsd")
- NULL =
File::NULL
Class Method Summary
collapse
Class Method Details
.app_cache(custom_path = nil) ⇒ Object
342
343
344
345
|
# File 'lib/bundler.rb', line 342
def app_cache(custom_path = nil)
path = custom_path || root
Pathname.new(path).join(Bundler.settings.app_cache_path)
end
|
.app_config_path ⇒ Object
328
329
330
331
332
333
334
335
336
337
338
339
340
|
# File 'lib/bundler.rb', line 328
def app_config_path
if app_config = ENV["BUNDLE_APP_CONFIG"]
app_config_pathname = Pathname.new(app_config)
if app_config_pathname.absolute?
app_config_pathname
else
app_config_pathname.expand_path(root)
end
else
root.join(".bundle")
end
end
|
.auto_install ⇒ Object
Automatically install dependencies if settings[:auto_install] exists. This is set through config cmd ‘bundle config set –global auto_install 1`.
Note that this method ‘nil`s out the global Definition object, so it should be called first, before you instantiate anything like an `Installer` that’ll keep a reference to the old one instead.
182
183
184
185
186
187
188
189
190
191
192
193
|
# File 'lib/bundler.rb', line 182
def auto_install
return unless Bundler.settings[:auto_install]
begin
definition.specs
rescue GemNotFound, GitError
ui.info "Automatically installing missing gems."
reset!
CLI::Install.new({}).run
reset!
end
end
|
.auto_switch ⇒ Object
172
173
174
|
# File 'lib/bundler.rb', line 172
def auto_switch
self_manager.restart_with_locked_bundler_if_needed
end
|
.bin_path ⇒ Object
Returns absolute location of where binstubs are installed to.
119
120
121
122
123
124
125
126
|
# File 'lib/bundler.rb', line 119
def bin_path
@bin_path ||= begin
path = Bundler.settings[:bin] || "bin"
path = Pathname.new(path).expand_path(root).expand_path
mkdir_p(path)
path
end
end
|
.bundle_path ⇒ Object
Returns absolute path of where gems are installed on the filesystem.
101
102
103
|
# File 'lib/bundler.rb', line 101
def bundle_path
@bundle_path ||= Pathname.new(configured_bundle_path.path).expand_path(root)
end
|
.bundler_major_version ⇒ Object
6
7
8
|
# File 'lib/bundler/version.rb', line 6
def self.bundler_major_version
@bundler_major_version ||= gem_version.segments.first
end
|
.clean_env ⇒ Object
367
368
369
370
371
372
|
# File 'lib/bundler.rb', line 367
def clean_env
removed_message =
"`Bundler.clean_env` has been removed in favor of `Bundler.unbundled_env`. " \
"If you instead want the environment before bundler was originally loaded, use `Bundler.original_env`"
Bundler::SharedHelpers.feature_removed!(removed_message)
end
|
.clean_exec(*args) ⇒ Object
423
424
425
426
427
428
|
# File 'lib/bundler.rb', line 423
def clean_exec(*args)
removed_message =
"`Bundler.clean_exec` has been removed in favor of `Bundler.unbundled_exec`. " \
"If you instead want to exec to a command in the environment before bundler was originally loaded, use `Bundler.original_exec`"
Bundler::SharedHelpers.feature_removed!(removed_message)
end
|
.clean_system(*args) ⇒ Object
406
407
408
409
410
411
|
# File 'lib/bundler.rb', line 406
def clean_system(*args)
removed_message =
"`Bundler.clean_system` has been removed in favor of `Bundler.unbundled_system`. " \
"If you instead want to run the command in the environment before bundler was originally loaded, use `Bundler.original_system`"
Bundler::SharedHelpers.feature_removed!(removed_message)
end
|
.clear_gemspec_cache ⇒ Object
539
540
541
|
# File 'lib/bundler.rb', line 539
def clear_gemspec_cache
@gemspec_cache = {}
end
|
87
88
89
|
# File 'lib/bundler.rb', line 87
def configure
@configure ||= configure_gem_home_and_path
end
|
590
591
592
593
594
595
596
597
|
# File 'lib/bundler.rb', line 590
def configure_custom_gemfile(custom_gemfile = nil)
custom_gemfile ||= Bundler.settings[:gemfile]
if custom_gemfile && !custom_gemfile.empty?
Bundler::SharedHelpers.set_env "BUNDLE_GEMFILE", File.expand_path(custom_gemfile)
reset_settings_and_root!
end
end
|
584
585
586
587
588
|
# File 'lib/bundler.rb', line 584
def configure_gem_home_and_path(path = bundle_path)
configure_gem_path
configure_gem_home(path)
Bundler.rubygems.clear_paths
end
|
114
115
116
|
# File 'lib/bundler.rb', line 114
def configured_bundle_path
@configured_bundle_path ||= Bundler.settings.path.tap(&:validate!)
end
|
.create_bundle_path ⇒ Object
105
106
107
108
109
110
111
112
|
# File 'lib/bundler.rb', line 105
def create_bundle_path
mkdir_p(bundle_path) unless bundle_path.exist?
@bundle_path = bundle_path.realpath
rescue Errno::EEXIST
raise PathError, "Could not install to path `#{bundle_path}` " \
"because a file already exists at that path. Either remove or rename the file so the directory can be created."
end
|
Returns current version of Ruby
9
10
11
|
# File 'lib/bundler/current_ruby.rb', line 9
def self.current_ruby
@current_ruby ||= CurrentRuby.new
end
|
.default_bundle_dir ⇒ Object
.default_gemfile ⇒ Object
.default_lockfile ⇒ Object
.definition(unlock = nil, lockfile = default_lockfile) ⇒ Bundler::Definition
Returns an instance of Bundler::Definition for given Gemfile and lockfile
231
232
233
234
235
236
237
|
# File 'lib/bundler.rb', line 231
def definition(unlock = nil, lockfile = default_lockfile)
@definition = nil if unlock
@definition ||= begin
configure
Definition.build(default_gemfile, lockfile, unlock)
end
end
|
.environment ⇒ Object
221
222
223
|
# File 'lib/bundler.rb', line 221
def environment
SharedHelpers.feature_removed! "Bundler.environment has been removed in favor of Bundler.load"
end
|
.find_executable(path) ⇒ Object
493
494
495
496
497
498
499
|
# File 'lib/bundler.rb', line 493
def find_executable(path)
extensions = RbConfig::CONFIG["EXECUTABLE_EXTS"]&.split
extensions = [RbConfig::CONFIG["EXEEXT"]] unless extensions&.any?
candidates = extensions.map {|ext| "#{path}#{ext}" }
candidates.find {|candidate| File.file?(candidate) && File.executable?(candidate) }
end
|
.frozen_bundle? ⇒ Boolean
239
240
241
242
243
244
|
# File 'lib/bundler.rb', line 239
def frozen_bundle?
frozen = Bundler.settings[:frozen]
return frozen unless frozen.nil?
Bundler.settings[:deployment]
end
|
.gem_version ⇒ Object
10
11
12
|
# File 'lib/bundler/version.rb', line 10
def self.gem_version
@gem_version ||= Gem::Version.create(VERSION)
end
|
440
441
442
|
# File 'lib/bundler.rb', line 440
def generic_local_platform
Gem::Platform.generic(local_platform)
end
|
.git_present? ⇒ Boolean
543
544
545
546
|
# File 'lib/bundler.rb', line 543
def git_present?
return @git_present if defined?(@git_present)
@git_present = Bundler.which("git")
end
|
.home ⇒ Object
306
307
308
|
# File 'lib/bundler.rb', line 306
def home
bundle_path.join("bundler")
end
|
.install_path ⇒ Object
310
311
312
|
# File 'lib/bundler.rb', line 310
def install_path
home.join("gems")
end
|
.load ⇒ Object
217
218
219
|
# File 'lib/bundler.rb', line 217
def load
@load ||= Runtime.new(root, definition)
end
|
.load_gemspec(file, validate = false) ⇒ Object
520
521
522
523
524
525
526
527
|
# File 'lib/bundler.rb', line 520
def load_gemspec(file, validate = false)
@gemspec_cache ||= {}
key = File.expand_path(file)
@gemspec_cache[key] ||= load_gemspec_uncached(file, validate)
@gemspec_cache[key]&.dup
end
|
.load_gemspec_uncached(file, validate = false) ⇒ Object
529
530
531
532
533
534
535
536
537
|
# File 'lib/bundler.rb', line 529
def load_gemspec_uncached(file, validate = false)
path = Pathname.new(file)
contents = read_file(file)
spec = eval_gemspec(path, contents)
return unless spec
spec.loaded_from = path.expand_path.to_s
Bundler.rubygems.validate(spec) if validate
spec
end
|
.original_env ⇒ Hash
Returns Environment present before Bundler was activated.
363
364
365
|
# File 'lib/bundler.rb', line 363
def original_env
ORIGINAL_ENV.clone
end
|
.original_exec(*args) ⇒ Object
Run a ‘Kernel.exec` to a subcommand with the environment present before Bundler was activated
419
420
421
|
# File 'lib/bundler.rb', line 419
def original_exec(*args)
with_original_env { Kernel.exec(*args) }
end
|
.original_system(*args) ⇒ Object
Run subcommand with the environment present before Bundler was activated
402
403
404
|
# File 'lib/bundler.rb', line 402
def original_system(*args)
with_original_env { Kernel.system(*args) }
end
|
.preferred_gemfile_name ⇒ Object
465
466
467
|
# File 'lib/bundler.rb', line 465
def preferred_gemfile_name
Bundler.settings[:init_gems_rb] ? "gems.rb" : "Gemfile"
end
|
.read_file(file) ⇒ Object
501
502
503
504
505
|
# File 'lib/bundler.rb', line 501
def read_file(file)
SharedHelpers.filesystem_access(file, :read) do
File.open(file, "r:UTF-8", &:read)
end
end
|
.require(*groups) ⇒ Object
Setups Bundler environment (see Bundler.setup) if it is not already set, and loads all gems from groups specified. Unlike ::setup, can be called multiple times with different groups (if they were allowed by setup).
Assuming Gemfile
gem 'first_gem', '= 1.0'
group :test do
gem 'second_gem', '= 1.0'
end
The code will work as follows:
Bundler.setup Bundler.require(:default) Bundler.require(:test)
213
214
215
|
# File 'lib/bundler.rb', line 213
def require(*groups)
setup(*groups).require(*groups)
end
|
.require_thor_actions ⇒ Object
4
5
6
|
# File 'lib/bundler/vendored_thor.rb', line 4
def self.require_thor_actions
require_relative "vendor/thor/lib/thor/actions"
end
|
.reset! ⇒ Object
552
553
554
555
556
|
# File 'lib/bundler.rb', line 552
def reset!
reset_paths!
Plugin.reset!
reset_rubygems!
end
|
.reset_paths! ⇒ Object
563
564
565
566
567
568
569
570
571
572
573
574
575
|
# File 'lib/bundler.rb', line 563
def reset_paths!
@bin_path = nil
@bundle_path = nil
@configure = nil
@configured_bundle_path = nil
@definition = nil
@load = nil
@locked_gems = nil
@root = nil
@settings = nil
@setup = nil
@user_home = nil
end
|
.reset_rubygems! ⇒ Object
577
578
579
580
581
582
|
# File 'lib/bundler.rb', line 577
def reset_rubygems!
return unless defined?(@rubygems) && @rubygems
rubygems.undo_replacements
rubygems.reset
@rubygems = nil
end
|
.reset_settings_and_root! ⇒ Object
558
559
560
561
|
# File 'lib/bundler.rb', line 558
def reset_settings_and_root!
@settings = nil
@root = nil
end
|
.root ⇒ Object
318
319
320
321
322
323
324
325
326
|
# File 'lib/bundler.rb', line 318
def root
@root ||= begin
SharedHelpers.root
rescue GemfileNotFound
bundle_dir = default_bundle_dir
raise GemfileNotFound, "Could not locate Gemfile or .bundle/ directory" unless bundle_dir
Pathname.new(File.expand_path("..", bundle_dir))
end
end
|
.ruby_scope ⇒ Object
256
257
258
|
# File 'lib/bundler.rb', line 256
def ruby_scope
"#{Bundler.rubygems.ruby_engine}/#{RbConfig::CONFIG["ruby_version"]}"
end
|
.rubygems ⇒ Object
453
454
455
|
# File 'lib/bundler/rubygems_integration.rb', line 453
def self.rubygems
@rubygems ||= RubygemsIntegration.new
end
|
.safe_load_marshal(data) ⇒ Object
507
508
509
510
511
512
513
514
515
516
517
518
|
# File 'lib/bundler.rb', line 507
def safe_load_marshal(data)
if Gem.respond_to?(:load_safe_marshal)
Gem.load_safe_marshal
begin
Gem::SafeMarshal.safe_load(data)
rescue Gem::SafeMarshal::Reader::Error, Gem::SafeMarshal::Visitors::ToRuby::Error => e
raise MarshalError, "#{e.class}: #{e.message}"
end
else
load_marshal(data, marshal_proc: SafeMarshal.proc)
end
end
|
.self_manager ⇒ Object
599
600
601
602
603
604
|
# File 'lib/bundler.rb', line 599
def self_manager
@self_manager ||= begin
require_relative "bundler/self_manager"
Bundler::SelfManager.new
end
end
|
.setup(*groups) ⇒ Object
Turns on the Bundler runtime. After Bundler.setup call, all load or require of the gems would be allowed only if they are part of the Gemfile or Ruby’s standard library. If the versions specified in Gemfile, only those versions would be loaded.
Assuming Gemfile
gem 'first_gem', '= 1.0'
group :test do
gem 'second_gem', '= 1.0'
end
The code using Bundler.setup works as follows:
require 'third_gem' require 'first_gem' Bundler.setup
require 'fourth_gem' require 'second_gem'
Bundler.setup can be called only once, all subsequent calls are no-op.
If groups list is provided, only gems from specified groups would be allowed (gems specified outside groups belong to special :default group).
To require all gems from Gemfile (or only some groups), see Bundler.require.
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
|
# File 'lib/bundler.rb', line 155
def setup(*groups)
return @setup if defined?(@setup) && @setup
configure_custom_gemfile
definition.validate_runtime!
SharedHelpers.print_major_deprecations!
if groups.empty?
@setup = load.setup
else
load.setup(*groups)
end
end
|
.simulated_version ⇒ Object
18
19
20
|
# File 'lib/bundler/version.rb', line 18
def self.simulated_version
@simulated_version ||= Bundler.settings[:simulate_version]
end
|
.specs_path ⇒ Object
314
315
316
|
# File 'lib/bundler.rb', line 314
def specs_path
bundle_path.join("specifications")
end
|
.system_bindir ⇒ Object
456
457
458
459
460
461
462
463
|
# File 'lib/bundler.rb', line 456
def system_bindir
Bundler.settings[:system_bindir] || Bundler.rubygems.gem_bindir
end
|
.tmp(name = Process.pid.to_s) ⇒ Object
347
348
349
350
|
# File 'lib/bundler.rb', line 347
def tmp(name = Process.pid.to_s)
Kernel.send(:require, "tmpdir")
Pathname.new(Dir.mktmpdir(["bundler", name]))
end
|
.ui ⇒ Object
91
92
93
|
# File 'lib/bundler.rb', line 91
def ui
(defined?(@ui) && @ui) || (self.ui = UI::Shell.new)
end
|
.unbundle_env! ⇒ Object
Remove all bundler-related variables from ENV
380
381
382
|
# File 'lib/bundler.rb', line 380
def unbundle_env!
ENV.replace(unbundle_env(ENV))
end
|
.unbundled_env ⇒ Hash
Returns Environment with all bundler-related variables removed.
375
376
377
|
# File 'lib/bundler.rb', line 375
def unbundled_env
unbundle_env(original_env)
end
|
.unbundled_exec(*args) ⇒ Object
Run a ‘Kernel.exec` to a subcommand in an environment with all bundler related variables removed
431
432
433
|
# File 'lib/bundler.rb', line 431
def unbundled_exec(*args)
with_env(unbundled_env) { Kernel.exec(*args) }
end
|
.unbundled_system(*args) ⇒ Object
Run subcommand in an environment with all bundler related variables removed
414
415
416
|
# File 'lib/bundler.rb', line 414
def unbundled_system(*args)
with_unbundled_env { Kernel.system(*args) }
end
|
.URI(uri) ⇒ Object
Returns a Bundler::URI object derived from the given uri, which may be a Bundler::URI string or an existing Bundler::URI object:
require 'bundler/vendor/uri/lib/uri'
uri = Bundler::URI('http://github.com/ruby/ruby')
Bundler::URI(uri)
You must require ‘bundler/vendor/uri/lib/uri’ to use this method.
911
912
913
914
915
916
917
918
919
920
|
# File 'lib/bundler/vendor/uri/lib/uri/common.rb', line 911
def URI(uri)
if uri.is_a?(Bundler::URI::Generic)
uri
elsif uri = String.try_convert(uri)
Bundler::URI.parse(uri)
else
raise ArgumentError,
"bad argument (expected Bundler::URI object or Bundler::URI string)"
end
end
|
.use_system_gems? ⇒ Boolean
469
470
471
|
# File 'lib/bundler.rb', line 469
def use_system_gems?
configured_bundle_path.use_system_gems?
end
|
.user_bundle_path(dir = "home") ⇒ Object
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
|
# File 'lib/bundler.rb', line 284
def user_bundle_path(dir = "home")
env_var, fallback = case dir
when "home"
["BUNDLE_USER_HOME", proc { Pathname.new(user_home).join(".bundle") }]
when "cache"
["BUNDLE_USER_CACHE", proc { user_bundle_path.join("cache") }]
when "config"
["BUNDLE_USER_CONFIG", proc { user_bundle_path.join("config") }]
when "plugin"
["BUNDLE_USER_PLUGIN", proc { user_bundle_path.join("plugin") }]
else
raise BundlerError, "Unknown user path requested: #{dir}"
end
Pathname.new(ENV.fetch(env_var, &fallback))
end
|
.user_cache ⇒ Object
302
303
304
|
# File 'lib/bundler.rb', line 302
def user_cache
user_bundle_path("cache")
end
|
.user_home ⇒ Object
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
|
# File 'lib/bundler.rb', line 260
def user_home
@user_home ||= begin
home = Bundler.rubygems.user_home
bundle_home = home ? File.join(home, ".bundle") : nil
warning = if home.nil?
"Your home directory is not set."
elsif !File.directory?(home)
"`#{home}` is not a directory."
elsif !File.writable?(home) && (!File.directory?(bundle_home) || !File.writable?(bundle_home))
"`#{home}` is not writable."
end
if warning
Bundler.ui.warn "#{warning}\n"
user_home = tmp_home_path
Bundler.ui.warn "Bundler will use `#{user_home}' as your home directory temporarily.\n"
user_home
else
Pathname.new(home)
end
end
end
|
.verbose_version ⇒ Object
14
15
16
|
# File 'lib/bundler/version.rb', line 14
def self.verbose_version
@verbose_version ||= "#{VERSION}#{simulated_version ? " (simulating Bundler #{simulated_version})" : ""}"
end
|
.which(executable) ⇒ Object
479
480
481
482
483
484
485
486
487
488
489
490
491
|
# File 'lib/bundler.rb', line 479
def which(executable)
executable_path = find_executable(executable)
return executable_path if executable_path
if (paths = ENV["PATH"])
quote = '"'
paths.split(File::PATH_SEPARATOR).find do |path|
path = path[1..-2] if path.start_with?(quote) && path.end_with?(quote)
executable_path = find_executable(File.expand_path(executable, path))
return executable_path if executable_path
end
end
end
|
.with_clean_env ⇒ Object
389
390
391
392
393
394
|
# File 'lib/bundler.rb', line 389
def with_clean_env
removed_message =
"`Bundler.with_clean_env` has been removed in favor of `Bundler.with_unbundled_env`. " \
"If you instead want the environment before bundler was originally loaded, use `Bundler.with_original_env`"
Bundler::SharedHelpers.feature_removed!(removed_message)
end
|
.with_friendly_errors ⇒ Object
.with_original_env ⇒ Object
Run block with environment present before Bundler was activated
385
386
387
|
# File 'lib/bundler.rb', line 385
def with_original_env
with_env(original_env) { yield }
end
|
.with_unbundled_env ⇒ Object
Run block with all bundler-related variables removed
397
398
399
|
# File 'lib/bundler.rb', line 397
def with_unbundled_env
with_env(unbundled_env) { yield }
end
|