Method: Bundler::FetchMetadata#matches_current_ruby?
- Defined in:
- lib/bundler/match_remote_metadata.rb
#matches_current_ruby? ⇒ Boolean
A fallback is included because the original version of the specification API didn’t include that field, so some marshalled specs in the index have it set to nil.
8 9 10 11 12 |
# File 'lib/bundler/match_remote_metadata.rb', line 8 def matches_current_ruby? @required_ruby_version ||= _remote_specification.required_ruby_version || Gem::Requirement.default super end |