Method: Bundler::InsecureInstallPathError#message
- Defined in:
- lib/bundler/errors.rb
#message ⇒ Object
243 244 245 246 247 |
# File 'lib/bundler/errors.rb', line 243 def "Bundler cannot reinstall #{@name} because there's a previous installation of it at #{@path} that is unsafe to remove.\n" \ "The parent of #{@path} is world-writable and does not have the sticky bit set, making it insecure to remove due to potential vulnerabilities.\n" \ "Please change the permissions of #{File.dirname(@path)} or choose a different install path." end |