Class: Bundler::CLI::Config::Base
- Inherits:
-
Object
- Object
- Bundler::CLI::Config::Base
- Defined in:
- lib/bundler/cli/config.rb
Instance Attribute Summary collapse
-
#name ⇒ Object
readonly
Returns the value of attribute name.
-
#options ⇒ Object
readonly
Returns the value of attribute options.
-
#scope ⇒ Object
readonly
Returns the value of attribute scope.
-
#thor ⇒ Object
readonly
Returns the value of attribute thor.
-
#value ⇒ Object
readonly
Returns the value of attribute value.
Instance Method Summary collapse
- #confirm(name) ⇒ Object
- #confirm_all ⇒ Object
- #explicit_scope? ⇒ Boolean
-
#initialize(options, name, value, thor) ⇒ Base
constructor
A new instance of Base.
- #message ⇒ Object
- #new_value ⇒ Object
- #run ⇒ Object
- #show_pretty_values_for(setting) ⇒ Object
- #validate_scope! ⇒ Object
- #warn_unused_scope(msg) ⇒ Object
Constructor Details
#initialize(options, name, value, thor) ⇒ Base
Returns a new instance of Base.
62 63 64 65 66 67 68 69 |
# File 'lib/bundler/cli/config.rb', line 62 def initialize(, name, value, thor) @options = @name = name value = Array(value) @value = value.empty? ? nil : value.join(" ") @thor = thor validate_scope! end |
Instance Attribute Details
#name ⇒ Object (readonly)
Returns the value of attribute name.
60 61 62 |
# File 'lib/bundler/cli/config.rb', line 60 def name @name end |
#options ⇒ Object (readonly)
Returns the value of attribute options.
60 61 62 |
# File 'lib/bundler/cli/config.rb', line 60 def @options end |
#scope ⇒ Object (readonly)
Returns the value of attribute scope.
60 61 62 |
# File 'lib/bundler/cli/config.rb', line 60 def scope @scope end |
#thor ⇒ Object (readonly)
Returns the value of attribute thor.
60 61 62 |
# File 'lib/bundler/cli/config.rb', line 60 def thor @thor end |
#value ⇒ Object (readonly)
Returns the value of attribute value.
60 61 62 |
# File 'lib/bundler/cli/config.rb', line 60 def value @value end |
Instance Method Details
#confirm(name) ⇒ Object
129 130 131 132 |
# File 'lib/bundler/cli/config.rb', line 129 def confirm(name) Bundler.ui.confirm "Settings for `#{name}` in order of priority. The top value will be used" show_pretty_values_for(name) end |
#confirm_all ⇒ Object
111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 |
# File 'lib/bundler/cli/config.rb', line 111 def confirm_all if @options[:parseable] thor.with_padding do Bundler.settings.all.each do |setting| val = Bundler.settings[setting] Bundler.ui.info "#{setting}=#{val}" end end else Bundler.ui.confirm "Settings are listed in order of priority. The top value will be used.\n" Bundler.settings.all.each do |setting| Bundler.ui.confirm setting show_pretty_values_for(setting) Bundler.ui.confirm "" end end end |
#explicit_scope? ⇒ Boolean
172 173 174 |
# File 'lib/bundler/cli/config.rb', line 172 def explicit_scope? @explicit_scope end |
#message ⇒ Object
143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 |
# File 'lib/bundler/cli/config.rb', line 143 def locations = Bundler.settings.locations(name) if @options[:parseable] "#{name}=#{new_value}" if new_value elsif scope == "global" if !locations[:local].nil? "Your application has set #{name} to #{locations[:local].inspect}. " \ "This will override the global value you are currently setting" elsif locations[:env] "You have a bundler environment variable for #{name} set to " \ "#{locations[:env].inspect}. This will take precedence over the global value you are setting" elsif !locations[:global].nil? && locations[:global] != value "You are replacing the current global value of #{name}, which is currently " \ "#{locations[:global].inspect}" end elsif scope == "local" && !locations[:local].nil? && locations[:local] != value "You are replacing the current local value of #{name}, which is currently " \ "#{locations[:local].inspect}" end end |
#new_value ⇒ Object
134 135 136 137 138 139 140 141 |
# File 'lib/bundler/cli/config.rb', line 134 def new_value pathname = Pathname.new(value) if name.start_with?("local.") && pathname.directory? pathname..to_s else value end end |
#run ⇒ Object
71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 |
# File 'lib/bundler/cli/config.rb', line 71 def run unless name warn_unused_scope "Ignoring --#{scope}" confirm_all return end if [:delete] if !explicit_scope? || scope != "global" Bundler.settings.set_local(name, nil) end if !explicit_scope? || scope != "local" Bundler.settings.set_global(name, nil) end return end if value.nil? warn_unused_scope "Ignoring --#{scope} since no value to set was given" current_value = Bundler.settings[name] if [:parseable] if value = Bundler.settings[name] Bundler.ui.info("#{name}=#{value}") end else confirm(name) end if current_value.nil? exit 1 else return end end Bundler.ui.info() if Bundler.settings.send("set_#{scope}", name, new_value) end |
#show_pretty_values_for(setting) ⇒ Object
164 165 166 167 168 169 170 |
# File 'lib/bundler/cli/config.rb', line 164 def show_pretty_values_for(setting) thor.with_padding do Bundler.settings.pretty_values_for(setting).each do |line| Bundler.ui.info line end end end |
#validate_scope! ⇒ Object
183 184 185 186 187 188 189 190 191 192 193 194 195 196 |
# File 'lib/bundler/cli/config.rb', line 183 def validate_scope! @explicit_scope = true scopes = %w[global local].select {|s| [s] } case scopes.size when 0 @scope = inside_app? ? "local" : "global" @explicit_scope = false when 1 @scope = scopes.first else raise InvalidOption, "The options #{scopes.join " and "} were specified. Please only use one of the switches at a time." end end |