Class: Bundler::Plugin::UnloadedSource
- Inherits:
-
Object
- Object
- Bundler::Plugin::UnloadedSource
- Includes:
- API::Source
- Defined in:
- lib/bundler/plugin/unloaded_source.rb
Overview
Stands in for a source handled by a plugin that is not loaded yet, so that the lockfile can still be parsed during the plugin install pass, and by external tools reading a lockfile without the plugin installed.
Instance Attribute Summary
Attributes included from API::Source
#checksum_store, #dependency_names, #name, #options, #uri
Instance Method Summary collapse
-
#==(other) ⇒ Object
(also: #eql?)
Unlike real plugin sources, where the handling class encodes the source type, all unloaded sources share this class, so the type must be compared explicitly.
- #hash ⇒ Object
Methods included from API::Source
#add_dependency_names, #app_cache_dirname, #app_cache_path, #cache, #cached!, #can_lock?, #double_check_for, #download, #fetch_gemspec_files, #gem_install_dir, #include?, #initialize, #install, #install_path, #installed?, #local!, #options_to_lock, #post_install, #remote!, #root, #spec_names, #specs, #to_lock, #to_s, #unlock!, #unmet_deps, #uri_hash
Instance Method Details
#==(other) ⇒ Object Also known as: eql?
Unlike real plugin sources, where the handling class encodes the source type, all unloaded sources share this class, so the type must be compared explicitly.
14 15 16 |
# File 'lib/bundler/plugin/unloaded_source.rb', line 14 def ==(other) super && ["type"] == other.["type"] end |
#hash ⇒ Object
20 21 22 |
# File 'lib/bundler/plugin/unloaded_source.rb', line 20 def hash [super, ["type"]].hash end |