Class: Bundler::Source
- Inherits:
-
Object
show all
- Defined in:
- lib/bundler/source.rb,
lib/bundler/source/git.rb,
lib/bundler/source/path.rb,
lib/bundler/source/gemspec.rb,
lib/bundler/source/metadata.rb,
lib/bundler/source/rubygems.rb,
lib/bundler/source/git/git_proxy.rb,
lib/bundler/source/path/installer.rb,
lib/bundler/source/rubygems/remote.rb,
lib/bundler/source/rubygems_aggregate.rb
Defined Under Namespace
Classes: Gemspec, Git, Metadata, Path, Rubygems, RubygemsAggregate
Instance Attribute Summary collapse
Instance Method Summary
collapse
Instance Attribute Details
#checksum_store ⇒ Object
Returns the value of attribute checksum_store.
14
15
16
|
# File 'lib/bundler/source.rb', line 14
def checksum_store
@checksum_store
end
|
#dependency_names ⇒ Object
Returns the value of attribute dependency_names.
12
13
14
|
# File 'lib/bundler/source.rb', line 12
def dependency_names
@dependency_names
end
|
Instance Method Details
#add_dependency_names(names) ⇒ Object
50
51
52
|
# File 'lib/bundler/source.rb', line 50
def add_dependency_names(names)
@dependency_names = Array(dependency_names) | Array(names)
end
|
#cached! ⇒ Object
46
|
# File 'lib/bundler/source.rb', line 46
def cached!; end
|
#can_lock?(spec) ⇒ Boolean
36
37
38
|
# File 'lib/bundler/source.rb', line 36
def can_lock?(spec)
spec.source == self
end
|
#dependency_names_to_double_check ⇒ Object
59
60
61
|
# File 'lib/bundler/source.rb', line 59
def dependency_names_to_double_check
specs.dependency_names
end
|
#double_check_for ⇒ Object
it’s possible that gems from one source depend on gems from some other source, so now we download gemspecs and iterate over those dependencies, looking for gems we don’t have info on yet.
57
|
# File 'lib/bundler/source.rb', line 57
def double_check_for(*); end
|
#download ⇒ Object
34
|
# File 'lib/bundler/source.rb', line 34
def download(*); end
|
#extension_cache_path(spec) ⇒ Object
83
84
85
86
87
88
89
90
|
# File 'lib/bundler/source.rb', line 83
def extension_cache_path(spec)
return unless Bundler.settings[:global_gem_cache]
return unless source_slug = extension_cache_slug(spec)
Bundler.user_cache.join(
"extensions", Gem::Platform.local.to_s, Bundler.ruby_scope,
source_slug, spec.full_name
)
end
|
#identifier ⇒ Object
75
76
77
|
# File 'lib/bundler/source.rb', line 75
def identifier
to_s
end
|
#include?(other) ⇒ Boolean
67
68
69
|
# File 'lib/bundler/source.rb', line 67
def include?(other)
other == self
end
|
#inspect ⇒ Object
71
72
73
|
# File 'lib/bundler/source.rb', line 71
def inspect
"#<#{self.class}:0x#{object_id} #{self}>"
end
|
#local! ⇒ Object
42
|
# File 'lib/bundler/source.rb', line 42
def local!; end
|
#local_only! ⇒ Object
44
|
# File 'lib/bundler/source.rb', line 44
def local_only!; end
|
#path? ⇒ Boolean
79
80
81
|
# File 'lib/bundler/source.rb', line 79
def path?
instance_of?(Bundler::Source::Path)
end
|
#prefer_local! ⇒ Object
40
|
# File 'lib/bundler/source.rb', line 40
def prefer_local!; end
|
#remote! ⇒ Object
48
|
# File 'lib/bundler/source.rb', line 48
def remote!; end
|
#spec_names ⇒ Object
63
64
65
|
# File 'lib/bundler/source.rb', line 63
def spec_names
specs.spec_names
end
|
#unmet_deps ⇒ Object
16
17
18
|
# File 'lib/bundler/source.rb', line 16
def unmet_deps
specs.unmet_dependency_names
end
|
#version_message(spec, locked_spec = nil) ⇒ Object
20
21
22
23
24
25
26
27
28
29
30
31
32
|
# File 'lib/bundler/source.rb', line 20
def version_message(spec, locked_spec = nil)
message = "#{spec.name} #{spec.version}"
message += " (#{spec.platform})" if spec.platform != Gem::Platform::RUBY && !spec.platform.nil?
if locked_spec
locked_spec_version = locked_spec.version
if locked_spec_version && spec.version != locked_spec_version
message += Bundler.ui.add_color(" (was #{locked_spec_version})", version_color(spec.version, locked_spec_version))
end
end
message
end
|