Method: Gem::Net::HTTP::Persistent::TimedStackMulti.hash_of_arrays
- Defined in:
- lib/bundler/vendor/net-http-persistent/lib/net/http/persistent/timed_stack_multi.rb
.hash_of_arrays ⇒ Object
Returns a new hash that has arrays for keys
Using a class method to limit the bindings referenced by the hash’s default_proc
9 10 11 |
# File 'lib/bundler/vendor/net-http-persistent/lib/net/http/persistent/timed_stack_multi.rb', line 9 def self.hash_of_arrays # :nodoc: Hash.new { |h,k| h[k] = [] } end |