Method: Gem::Net::HTTP::Persistent#verify_hostname
- Defined in:
- lib/bundler/vendor/net-http-persistent/lib/net/http/persistent.rb
#verify_hostname ⇒ Object
HTTPS verify_hostname.
If a client sets this to true and enables SNI with SSLSocket#hostname=, the hostname verification on the server certificate is performed automatically during the handshake using OpenSSL::SSL.verify_certificate_identity().
You can set verify_hostname as true to use hostname verification during the handshake.
NOTE: This works with Ruby > 3.0.
467 468 469 |
# File 'lib/bundler/vendor/net-http-persistent/lib/net/http/persistent.rb', line 467 def verify_hostname @verify_hostname end |